package net.black_cow.opennip.ui.console;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import net.black_cow.opennip.opennipcore.NipManager;
import net.black_cow.opennip.opennipcore.NipObserver;
import net.black_cow.opennip.opennipcore.NipPlayer;
import net.black_cow.opennip.opennipcore.NipStone;
import net.black_cow.opennip.opennipcore.NipStrategy;
import net.black_cow.opennip.opennipcore.NipTable;
import net.black_cow.opennip.opennipcore.NipStone.Color;

/**
 * OpenNipCUINX.
 * @author percy
 *
 */
public class ConsoleUI implements NipObserver, NipStrategy {

	private static NipManager manager;
	
	private BufferedReader br;
	
	public static void main(String[] args) {
		NipPlayer first = new NamedPlayer("");
		NipPlayer second = new NamedPlayer("");
		
		manager = new NipManager(first, second);
		
		ConsoleUI ui = new ConsoleUI();
		
		first.setStrategy(ui);
		second.setStrategy(ui);
		
		manager.addObservers(ui);
		manager.play();
		
		ui.destruct();
	}
	
	private ConsoleUI() {
		br = new BufferedReader(new InputStreamReader(System.in));
	}
	
	private void destruct() {
		if(br != null) {
			try {
				br.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	@Override
	public void onBothPlayersCannotPutStone() {
		System.out.println("Ґ΂uꏊ܂B");		
	}

	@Override
	public void onGameEnd(NipPlayer winner, int winnerScore, int loserScore) {
		System.out.println("Q[IB");
		if(winner != null) {
			System.out.println("ҁ@" + winner);
			System.out.println("XRA@" + winnerScore);
		} else {
			System.out.println("B");
			System.out.println("XRA@" + winnerScore);
		}
	}

	@Override
	public void onGameStart() {
		System.out.println("Q[X^[gI");
		printTable();
	}

	@Override
	public void onPlayerTurn(NipPlayer player) {
		System.out.println(player + "̃^[B" + player.getColor());
		printTable();
	}

	@Override
	public void onNewStone(int row, int column, Color color) {
		System.out.println("V΂u܂B");
		printTable();
	}

	@Override
	public void onNoPlaceToPut(NipPlayer player) {
		System.out.println(player + "͒uꏊ܂B");
	}

	@Override
	public void onStoneColorChanged(int row, int column, Color color) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void onStoneColorChangedFinished() {
		System.out.println("΂̐FςI܂B");
	}

	@Override
	public int[] decide(NipTable table, NipStone stone) {
		System.out.println("΂uꏊw肵ĂiF3 4jB");
		String in;
		int[] ret = new int[2];
		while(true) {
			if(!(in = readLine()).matches("[0-7] [0-7]")) {
				System.out.println("͂słB");
				continue;
			}
			ret[0] = Integer.parseInt(in.substring(0, 1));
			ret[1] = Integer.parseInt(in.substring(2, 3));
			
			if(table.canPutStoneAt(ret[0], ret[1], stone)) {
				break;
			}
			System.out.println("w肳ꂽʒuɂ͒u܂B");
		}
		return ret;
	}
	
	private void printTable() {
		System.out.println("@ OPQRSTUV@");
		System.out.println("@ QQQQQQQ@");
		for(int i=0; i<=7; i++) {
			System.out.print(i + "b");
			for(int j=0; j<=7; j++) {
				if(((i==0 || i==7) && (j<=1||j>=6)) || ((i==1 || i==6) && (j==0||j==7)) ) {
					System.out.print("@");
				}else {				
					Color color = manager.getCellColor(i, j);
					System.out.print(convertCellColorToString(color));
				}
			}
			System.out.println("b");
		}
		System.out.println("@PPPPPPP@");
	}
	
	private String convertCellColorToString(Color color) {
		if(color == null) return "@";
		if(color == Color.BLACK) return "";
		return "";
	}
	
	private String readLine() {
		try {
			return br.readLine();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}

}
