package net.black_cow.opennip.ui.swing;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.black_cow.opennip.opennipcore.NipManager;
import net.black_cow.opennip.opennipcore.NipPlayer;
import net.black_cow.opennip.opennipcore.NipStone;
import net.black_cow.opennip.opennipcore.NipStrategy;
import net.black_cow.opennip.strategy.MixStrategy;
import net.black_cow.opennip.strategy.MouseClickStrategy;
import net.black_cow.opennip.strategy.OneStepMaximizeStrategy;
import net.black_cow.opennip.strategy.RandomStrategy;

/**
 * UIǗNX.
 * @author percy
 *
 */
public class OpenNipUI {

	private MouseClickInfo info = new MouseClickInfo();
	private static NipManager manager;

	private JFrame frame = new JFrame(UIConstant.FRAME_TITLE);
	private JPanel boardPanel = new JPanel();
	private BoardLabel board;
	private JPanel informationPanel = new JPanel();
	private JLabel information = new JLabel();
	private JPanel messagePanel = new JPanel();
	private JLabel message = new JLabel();

	private int whiteScore;
	private int blackScore;

	private MessageDialog dialog = new MessageDialog(frame);

	public static void main(String[] args) {

		OpenNipUI ui = new OpenNipUI();
		ui.initGUI();

		NipPlayer first = new NipPlayer();
		NipPlayer second = new NipPlayer();

		ui.showDialog(UIConstant.TITLE_MESSAGE, UIConstant.TITLE_MESSAGE_LIFE_TIME, UIConstant.TITLE_MESSAGE_FONT, UIConstant.TITLE_MESSAGE_COLOR);

		do {
			first.setStrategy(ui.askStrategy(UIConstant.ASK_FIRST_PLAYER_TEXT));
			second.setStrategy(ui.askStrategy(UIConstant.ASK_SECOND_PLAYER_TEXT));
			manager = new NipManager(first, second);
			manager.addObservers(new ConcreteNipObserver(ui));
			manager.play();
		} while(ui.askReplay());

	}

	/**
	 * RXgN^.
	 */
	public OpenNipUI() {
		board = new BoardLabel(new ImageIcon(this.getClass().getClassLoader().getResource(UIConstant.BOARD_GIF_PATH)), info);
	}

	/*
	 * bZ[Wpl̃eLXgݒ肷B
	 */
	void setMessage(String message) {
		this.message.setText(message);
	}

	/*
	 * CtH[Vpl̃eLXgݒ肷B
	 */
	void refreshInformation() {
		information.setText(UIConstant.TEXT_AREA_BLACK + " " + blackScore + " " + UIConstant.TEXT_AREA_CENTER + " " + whiteScore + " " + UIConstant.TEXT_AREA_WHITE);
	}

	/*
	 * NipManagerCX^XTablȅ󂯎ABoardPanelƃCtH[Vpl̏XVB
	 * Q[JnɕK͎sKvB
	 */
	void refresh() {
		board.refresh(manager);
		this.whiteScore = manager.getStoneCount(NipStone.Color.WHITE);
		this.blackScore = manager.getStoneCount(NipStone.Color.BLACK);
		refreshInformation();
	}

	/*
	 * w肳ꂽZɎw肳ꂽF`悷B
	 */
	void drawBoard(int row, int column, NipStone.Color color) {
		board.paint(row, column, color);
	}

	/*
	 * w肳ꂽF̃XRAPJEgAbvB
	 */
	void incrementScore(NipStone.Color color) {
		if(color.equals(NipStone.Color.BLACK)) {
			blackScore++;
		} else {
			whiteScore++;
		}
	}

	/*
	 * w肳ꂽF̃XRAPJEg_EB
	 */
	void decrementScore(NipStone.Color color) {
		if(color.equals(NipStone.Color.BLACK)) {
			blackScore--;
		} else {
			whiteScore--;
		}
	}

	/*
	 * GUȈsB
	 */
	private void initGUI() {
		setBoardPanel();
		setInformationPanel();
		setMessagePanel();
		setFrameFactors();
	}

	/*
	 * boardPanel̐ݒsB
	 */
	private void setBoardPanel() {
		boardPanel.setBackground(UIConstant.BOARD_PANEL_COLOR);
		boardPanel.add(board);
		boardPanel.setPreferredSize(UIConstant.BOARD_PANEL_SIZE);
	}

	/*
	 * informationPanel̊eݒsB
	 */
	private void setInformationPanel() {
		information.setText(UIConstant.TEXT_AREA_WHITE + " 0 " + UIConstant.TEXT_AREA_CENTER + " 0 " + UIConstant.TEXT_AREA_BLACK);
		informationPanel.setBackground(UIConstant.INFORMATION_PANEL_COLOR);
		information.setFont(UIConstant.INFORMATION_FONT);
		information.setHorizontalAlignment(JLabel.CENTER);
		information.setVerticalAlignment(JLabel.CENTER);
		information.setPreferredSize(UIConstant.INFORMATION_PANEL_SIZE);
		informationPanel.setLayout(new GridLayout(1, 1));
		informationPanel.setPreferredSize(UIConstant.INFORMATION_PANEL_SIZE);
		informationPanel.add(information);
	}

	/*
	 * messagePanel̊eݒsB
	 */
	private void setMessagePanel() {
		messagePanel.setBackground(UIConstant.MESSAGE_PANEL_COLOR);
		message.setFont(UIConstant.MESSAGE_FONT);
		message.setHorizontalAlignment(JLabel.CENTER);
		message.setVerticalAlignment(JLabel.CENTER);
		message.setPreferredSize(UIConstant.MESSAGE_PANEL_SIZE);
		messagePanel.setLayout(new GridLayout(1, 1));
		messagePanel.add(message);
		messagePanel.setPreferredSize(UIConstant.MESSAGE_PANEL_SIZE);
	}

	/*
	 * frameCX^X̊eݒsB
	 */
	private void setFrameFactors() {
		frame.setResizable(false);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.getContentPane().setLayout(new BoxLayout(frame.getContentPane(), BoxLayout.Y_AXIS));
		frame.getContentPane().add(boardPanel);
		frame.getContentPane().add(informationPanel);
		frame.getContentPane().add(messagePanel);
		frame.pack();
		frame.setLocationRelativeTo(null);
		frame.setVisible(true);
	}

	void showDialog(String message, long lifeTime, Font font, Color textColor) {
		dialog.setText(message, font, textColor);
		dialog.setVisible(true);
		try {
			Thread.sleep(lifeTime);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		dialog.setVisible(false);
	}

	private NipStrategy askStrategy(String message) {
		JButton man = new JButton(UIConstant.MAN_BUTTON_TEXT);
		man.setPreferredSize(UIConstant.BUTTON_SIZE);
		JButton weak = new JButton(UIConstant.WEAK_BUTTON_TEXT);
		weak.setPreferredSize(UIConstant.BUTTON_SIZE);
		JButton mid = new JButton(UIConstant.MID_BUTTON_TEXT);
		mid.setPreferredSize(UIConstant.BUTTON_SIZE);
		JButton strong = new JButton(UIConstant.STRONG_BUTTON_TEXT);
		strong.setPreferredSize(UIConstant.BUTTON_SIZE);
		MessageModalDialog dialog = new MessageModalDialog(frame, man, weak, mid, strong);
		dialog.setText(message, UIConstant.DESCRIPTION_FONT, null);
		dialog.setVisible(true);

		Object reply = dialog.getReply();
		if(reply == man) {
			return new MouseClickStrategy(info);
		} else if(reply == weak) {
			return new RandomStrategy(UIConstant.CPU_WAIT_TIME);
		} else if(reply == mid) {
			return new OneStepMaximizeStrategy(UIConstant.CPU_WAIT_TIME);
		} else {
			return new MixStrategy(5, UIConstant.CPU_WAIT_TIME);
		}
	}

	private boolean askReplay() {
		JButton yes = new JButton(UIConstant.YES_BUTTON_TEXT);
		yes.setPreferredSize(UIConstant.BUTTON_SIZE);
		JButton no = new JButton(UIConstant.NO_BUTTON_TEXT);
		no.setPreferredSize(UIConstant.BUTTON_SIZE);
		MessageModalDialog dialog = new MessageModalDialog(frame, yes, no);
		dialog.setText(UIConstant.ASK_REPLAY_MESSAGE, UIConstant.DESCRIPTION_FONT, null);
		dialog.setVisible(true);
		return dialog.getReply() == yes ? true : false;
	}
}
