package net.black_cow.opennip.opennipcore;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * jbṽe[u\NXB
 *
 * @author percy
 *
 */
public class NipTable {

	private class Cell {
		private NipStone stone = null;

		private Map<Position, Cell> neighbors = new HashMap<Position, Cell>();

		private int row;
		private int column;

		Cell(int row, int column) {
			this.row = row;
			this.column = column;
		}

		private void putStone(NipStone stone) {
			if (stone == null)
				throw new IllegalArgumentException("null͋e܂B");
			if (this.stone != null)
				throw new RuntimeException("łɐ΂uĂ܂B");
			this.stone = stone;
		}

		private boolean hasStone() {
			return stone != null;
		}

		private NipStone getStone() {
			return stone;
		}

		private void setNeighbor(Position position, Cell cell) {
			neighbors.put(position, cell);
		}

		private Cell neighbor(Position position) {
			return neighbors.get(position);
		}

		private int getRowIndex() {
			return row;
		}

		private int getColumnIndex() {
			return column;
		}
	}

	private enum Position {
		TOP(0, -1), TOP_RIGHT(1, -1), RIGHT(1, 0), BOTTOM_RIGHT(1, 1), BOTTOM(0, 1),
		BOTTOM_LEFT(-1, 1), LEFT(-1, 0), TOP_LEFT(-1, -1), CIRCLE_RIGHT(0, 0), CIRCLE_LEFT(0,0);

		private final int i, j;

		private Position(int i, int j) {
			this.i = i;
			this.j = j;
		}
	}

	private Cell[][] cells = new Cell[8][8];

	private List<NipObserver> observers = new ArrayList<NipObserver>();

	/*
	 * RXgN^.
	 */
	NipTable() {
		setCells();
		setNeighbors();
	}

	/*
	 * e[uB
	 */
	void initTable() {
		cells[3][3].putStone(new NipStone(NipStone.Color.WHITE));
		cells[3][4].putStone(new NipStone(NipStone.Color.BLACK));
		cells[4][3].putStone(new NipStone(NipStone.Color.BLACK));
		cells[4][4].putStone(new NipStone(NipStone.Color.WHITE));
	}

	/*
	 * zcellsɁACellCX^Xi[B
	 */
	private void setCells() {
		for (int i = 2; i <= 5; i++) {
			cells[0][i] = new Cell(0, i);
			cells[7][i] = new Cell(7, i);
		}
		for (int i = 1; i <= 6; i++) {
			cells[1][i] = new Cell(1, i);
			cells[6][i] = new Cell(6, i);
		}
		for(int i= 0; i<=7; i++) {
			for(int j=2; j<=5; j++) {
				cells[j][i] = new Cell(j, i);
			}
		}
	}

	/*
	 * eCellCX^XɁAאڂCellCX^Xݒ肷B
	 */
	private void setNeighbors() {
		for(int i=0; i<=7; i++) {
			for(int j=0; j<=7; j++) {
				Cell cell = getCell(i, j);
				for(Position pos : Position.values()) {
					if(cell != null && pos != Position.CIRCLE_RIGHT && pos != Position.CIRCLE_LEFT) {
						cell.setNeighbor(pos, getCell(i + pos.i, j + pos.j));
					}
				}
			}
		}
		cells[0][2].setNeighbor(Position.CIRCLE_RIGHT, cells[1][1]);
		for(int i=3; i<=5; i++) cells[0][i].setNeighbor(Position.CIRCLE_RIGHT, cells[0][i-1]);
		cells[1][6].setNeighbor(Position.CIRCLE_RIGHT, cells[0][5]);
		cells[2][7].setNeighbor(Position.CIRCLE_RIGHT, cells[1][6]);
		for(int i=3; i<=5; i++) cells[i][7].setNeighbor(Position.CIRCLE_RIGHT, cells[i-1][7]);
		cells[6][6].setNeighbor(Position.CIRCLE_RIGHT, cells[5][7]);
		cells[7][5].setNeighbor(Position.CIRCLE_RIGHT, cells[6][6]);
		for(int i=4; i>=2; i--) cells[7][i].setNeighbor(Position.CIRCLE_RIGHT, cells[7][i+1]);
		cells[6][1].setNeighbor(Position.CIRCLE_RIGHT, cells[7][2]);
		cells[5][0].setNeighbor(Position.CIRCLE_RIGHT, cells[6][1]);
		for(int i=4; i>=2; i--) cells[i][0].setNeighbor(Position.CIRCLE_RIGHT, cells[i+1][0]);
		cells[1][1].setNeighbor(Position.CIRCLE_RIGHT, cells[2][0]);

		for(int i=2; i<=4; i++) cells[0][i].setNeighbor(Position.CIRCLE_LEFT, cells[0][i+1]);
		cells[0][5].setNeighbor(Position.CIRCLE_LEFT, cells[1][6]);
		cells[1][6].setNeighbor(Position.CIRCLE_LEFT, cells[2][7]);
		for(int i=2; i<=4; i++) cells[i][7].setNeighbor(Position.CIRCLE_LEFT, cells[i+1][7]);
		cells[5][7].setNeighbor(Position.CIRCLE_LEFT, cells[6][6]);
		cells[6][6].setNeighbor(Position.CIRCLE_LEFT, cells[7][5]);
		for(int i=5; i>=3; i--) cells[7][i].setNeighbor(Position.CIRCLE_LEFT, cells[7][i-1]);
		cells[7][2].setNeighbor(Position.CIRCLE_LEFT, cells[6][1]);
		cells[6][1].setNeighbor(Position.CIRCLE_LEFT, cells[5][0]);
		for(int i=5; i>=3; i--) cells[i][0].setNeighbor(Position.CIRCLE_LEFT, cells[i-1][0]);
		cells[2][0].setNeighbor(Position.CIRCLE_LEFT, cells[1][1]);
		cells[1][1].setNeighbor(Position.CIRCLE_LEFT, cells[0][2]);
	}

	/*
	 * w肳ꂽʒuCellCX^XԂB
	 * ̈Ow肳ꂽꍇAnullԂB
	 */
	private Cell getCell(int row, int column) {
		if(row < 0 || row > 7) {
			return null;
		}
		if(row == 0 || row == 7) {
			if(column < 2 || column > 5) {
				return null;
			}
		}
		if(row == 1 || row == 6) {
			if(column < 1 || column > 6) {
				return null;
			}
		}
		if(column < 0 || column > 7) {
			return null;
		}
		return cells[row][column];
	}

	/*
	 * w肳ꂽCellɁAw肳ꂽStoneCX^XuB
	 * uƂɂ苲܂ꂽ΂̐FςB
	 */
	void putStoneAt(int row, int column, NipStone stone) {
		Cell cell = getCell(row, column);
		if(cell == null) throw new IllegalArgumentException("Z̎w肪słB");
		if(!canPutStoneAt(cell, stone)) throw new IllegalArgumentException("̃Zɂ͒u܂B");

		List<Position> list = new ArrayList<Position>();
		for(Position position : Position.values()) {
			if(scanLine(cell, stone, position))
				list.add(position);
		}

		cell.putStone(stone);
		notifyObserversNewStone(row, column, stone.getColor());
		for(Position position : list) {
			reverseStones(cell.neighbor(position), stone, position);
		}
		notifyObserversStoneColorChangedFinished();
	}

	/*
	 * w肳ꂽiPositionj̐΂̐FςĂB
	 */
	private void reverseStones(Cell cell, NipStone stone, Position position) {
		if(cell.getStone().equals(stone)) return;

		cell.getStone().changeColor();
		notifyObserversStoneColorChanged(cell.getRowIndex(), cell.getColumnIndex(), cell.getStone().getColor());
		reverseStones(cell.neighbor(position), stone, position);
	}

	/*
	 * w肳ꂽStoneCX^XAe[ûꂩCellɒuƂł邩ׂB
	 */
	boolean canPutStone(NipStone stone) {
		for(Cell[] column : cells) {
			for(Cell cell : column) {
				if(cell != null && this.canPutStoneAt(cell, stone))
					return true;
			}
		}
		return false;
	}

	/**
	 * w肳ꂽCellɁAw肳ꂽStoneCX^XuƂł邩ׂB
	 * @param row s̈ʒu
	 * @param column ̈ʒu
	 * @param stone 
	 * @return uƂłꍇtrueAuƂłȂA͎w肳ꂽZȂꍇfalse
	 */
	public boolean canPutStoneAt(int row, int column, NipStone stone) {
		Cell cell = getCell(row, column);
		return canPutStoneAt(cell, stone);
	}

	/*
	 * w肳ꂽCellɁAw肳ꂽStoneCX^XuƂł邩ׂB
	 */
	private boolean canPutStoneAt(Cell cell, NipStone stone) {
		if(cell == null) return false;
		if(cell.hasStone()) return false;
		for(Position position : Position.values()) {
			if(scanLine(cell, stone, position)) return true;
		}
		return false;
	}

	/*
	 * w肳ꂽCellɁAw肳ꂽStoneCX^XuƂł邩Aw̕iPositionjɂĂ̂ݒׂB
	 */
	private boolean scanLine(Cell cell, NipStone stone, Position position) {
		Cell neighbor = cell.neighbor(position);
		if(neighbor != null && neighbor.hasStone() && !neighbor.getStone().equals(stone))
			return scanNeighbors(neighbor, position);

		return false;
	}

	/*
	 * w肳ꂽCellCX^X́Aw肳ꂽɁA΂A`ŁAقȂF̐΂邩𔻒肷B
	 * j
	 * true
	 * @false
	 */
	private boolean scanNeighbors(Cell cell, Position position) {
		Cell neighbor = cell.neighbor(position);
		if(neighbor == null || !neighbor.hasStone()) return false;
		if(cell.getStone().equals(neighbor.getStone())) return scanNeighbors(neighbor, position);
		return true;
	}

	/*
	 * e[uł܂΂uĂȂZ̐ԂB
	 */
	int getBlankCellCount() {
		int count = 0;
		for(Cell[] column : cells) {
			for(Cell cell : column) {
				if(cell != null && !cell.hasStone()) count++;
			}
		}
		return count;
	}

	/*
	 * w肳ꂽF̐΂̐ԂB
	 */
	int getStoneCount(NipStone.Color color) {
		int count = 0;
		for(Cell[] column : cells) {
			for(Cell cell : column) {
				if(cell != null && cell.hasStone() && cell.getStone().getColor().equals(color))
					count++;
			}
		}
		return count;
	}

	/**
	 * w肳ꂽZɂ΂̐FԂB
	 * @param row s̈ʒu
	 * @param column ̈ʒu
	 * @return ΂̐Fi΂ꍇnullj
	 */
	public NipStone.Color getCellColor(int row, int column) {
		Cell cell = getCell(row, column);
		if(cell == null) throw new IllegalArgumentException("Z̎w肪słB");
		NipStone stone = cell.getStone();
		if(stone != null) return stone.getColor();

		return null;
	}

	/*
	 * IuU[oǉB
	 */
	void addObservers(NipObserver observer) {
		observers.add(observer);
	}

	/*
	 * V΂uꂽƂʒmB
	 */
	private void notifyObserversNewStone(int row, int column, NipStone.Color color) {
		for(NipObserver observer : observers) {
			observer.onNewStone(row, column, color);
		}
	}

	/*
	 * ΂̐FςƂʒmB
	 */
	private void notifyObserversStoneColorChanged(int row, int column, NipStone.Color color) {
		for(NipObserver observer : observers) {
			observer.onStoneColorChanged(row, column, color);
		}
	}

	/*
	 * ΂̐FSĕςIƂʒmB
	 */
	private void notifyObserversStoneColorChangedFinished() {
		for(NipObserver observer : observers) {
			observer.onStoneColorChangedFinished();
		}
	}

	/*
	 * IuU[oݒ肳ĂȂg̃Rs[ԂB
	 */
	NipTable getCopy() {
		NipTable table = new NipTable();
		for(int i=0; i<=7; i++) {
			for(int j=0; j<=7; j++) {
				Cell cell = getCell(i, j);
				if(cell != null && cell.hasStone())
					table.cells[i][j].putStone((NipStone) cell.getStone().clone());
			}
		}
		return table;
	}
}
